/*
 *  Kylin security firewall
 *
 *  Authors:  mengyuan <mengyuan@kylinos.cn>
 *
 *  Copyright (C) 2023,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef _KYSEC_FIREWALL_H
#define _KYSEC_FIREWALL_H

# ifdef  __cplusplus
extern "C" {
# endif

enum kysec_firewall_mode {
    FW_ALL_MODE,   // 同时启用公用与专用网络
    FW_PUBLIC,     // 公用网络
    FW_PRIVATE,    // 专用网络
    FW_OFF,        // 关闭防火墙
    FW_RESERVE     // 系统保留
};

enum kysec_firewall_policy {
    INBOUND_DENY_ALL,      // 入站阻止所有连接
    INBOUND_DENY,          // 入站阻止（默认）
    INBOUND_ALLOW,         // 入站允许
    OUTBOUND_DENY,         // 出站阻止
    OUTBOUND_ALLOW         // 出站允许（默认）
};

enum kysec_firewall_direction {
    FW_ALL_DIRECITON,    // 所有
    FW_INBOUND,          // 出站
    FW_OUTBOUND          // 入站
};

enum kysec_firewall_action {
    FW_ALLOW,           // 放行
    FW_DENY             // 阻止
};

enum kysec_firewall_status {
    FW_RULE_OFF,         // 禁用
    FW_RULE_ON           // 启用
};

typedef struct KYSEC_FW_RULE KYSEC_FW_RULE;

enum kysec_firewall_rule_item {
    FW_NAME,               // 规则名称
    FW_COMM,               // 应用（进程名称）
    FW_DIRECTION,          // 方向
    FW_MODE,               // 网络
    FW_PROTOCOL,           // 协议
    FW_LOCAL_IP,           // 本地IP
    FW_LOCAL_PORTS,        // 本地端口
    FW_REMOTE_IP,          // 远程IP
    FW_REMOTE_PORTS,       // 远程端口
    FW_ACTION,             // 操作
    FW_STATUS              // 状态
};

enum kysec_firewall_invalid_rule {
    RULE_EXIST_ERROR = -13,    // 规则已存在
    NULL_RULE_ERROR,           // 规则为空
    RULE_NAME_ERROR,           // 规则名为空
    DIRECTION_ERROR,           // 方向错误
    NET_MODE_ERROR,            // 网络模式错误
    ACTION_ERROR,              // 操作错误
    STATUS_ERROR,              // 状态错误
    PROTOCOL_ERROR,            // 协议错误
    LOCAL_IP_ERROR,            // 本地IP错误
    REMOTE_IP_ERROR,           // 远程IP错误
    LOCAL_PORT_ERROR,          // 本地端口错误
    REMOTE_PORT_ERROR,         // 远程端口错误
    NORMAL_ERROR               // 其他错误
};

/*
 * @brief 将当前网络连接对应的网卡与防火墙模式绑定
 * @param mode          网络模式
 * @param nic           当前网络连接对应的网卡名称
 * @return 配置成功返回0,失败返回-1
 */
int kysec_firewall_bind_network(int mode, char *nic);

/*
 * @brief 设置KSC防火墙网络模式的出、入站管控策略
 * @param mode          网络模式
 * @param in_policy     入站策略
 * @param out_policy    出站策略
 * @return 设置成功返回0,失败返回-1 
 */
int kysec_firewall_set_policy(int mode, int in_policy, int out_policy);

/*
 * @brief 获取KSC防火墙网络模式的出、入站管控策略
 * @param mode          网络模式
 * @param in_policy     用于保存获取的入站策略
 * @param out_policy    用于保存获取的出站策略
 * @return 获取成功返回0,失败返回-1 
 */
int kysec_firewall_get_policy(int mode, int *in_policy, int *out_policy);

/*
 * @brief 防火墙网络模式启用
 * @param mode          网络模式
 * @return 配置成功返回0,失败返回-1
 */
int kysec_firewall_enable_mode(int mode);

/*
 * @brief 防火墙网络模式禁用
 * @param mode          网络模式
 * @return 配置成功返回0,失败返回-1
 */
int kysec_firewall_disable_mode(int mode);

/*
 * @brief 获取当前KSC防火墙网络模式
 * @return 成功返回当前防火墙网络模式,失败返回-1 
 */
int kysec_firewall_get_mode();

/*
 * @brief 默认规则
 * （名称为default，应用为所有，方向为所有，网络为所有，
 * 协议为所有，本地、远程IP为任意IP，本地、远程端口为所有端口，操作为阻止）
 * @return 成功返回默认规则结构体指针,失败返回NULL
 * @attention 该接口自行分配内存，调用后需使用kysec_firewall_destory_rule释放内存
 */
KYSEC_FW_RULE *kysec_firewall_init_rule(void);

/*
 * @brief KSC防火墙规则字符串类型配置项赋值
 * @param rule          防火墙规则结构体指针
 * @param setting       待配置的字符串类型配置项,详见kysec_firewall_rule_item枚举
 * @param value         配置值，注意item与配置值类型对应
 * @return 成功返回0,失败返回-1 
 */
int kysec_firewall_set_rule_item(KYSEC_FW_RULE *rule, int item, void *value);

/*
 * @brief 释放防火墙规则结构体内存
 * @param rule          防火墙规则结构体指针
 */
void kysec_firewall_destory_rule(KYSEC_FW_RULE *rule);

/*
 * @brief 新建KSC防火墙规则
 * @param rule          防火墙规则结构体指针
 * @return 成功返回0
 *         失败返回值：
 *  RULE_EXIST_ERROR  (-13)  规则已存在
 *  NULL_RULE_ERROR  （-12）  规则为空
 *  RULE_NAME_ERROR  （-11）  规则名为空
 *  DIRECTION_ERROR  （-10）  方向错误
 *  NET_MODE_ERROR   （-9）   网络模式错误
 *  ACTION_ERROR     （-8）   操作错误
 *  STATUS_ERROR     （-7）   状态错误
 *  PROTOCOL_ERROR   （-6）   协议错误
 *  LOCAL_IP_ERROR   （-5）   本地IP错误
 *  REMOTE_IP_ERROR  （-4）   远程IP错误
 *  LOCAL_PORT_ERROR （-3）   本地端口错误
 *  REMOTE_PORT_ERROR（-2）   远程端口错误
 *  NORMAL_ERROR     （-1）   规则创建失败
 * @attention 请使用kysec_firewall_init_rule初始化KYSEC_FW_RULE结构体，
 * 使用kysec_firewall_set_rule_item接口设置KYSEC_FW_RULE字符串类型配置
 */
int kysec_firewall_create_rule(KYSEC_FW_RULE *rule);

/*
 * @brief 启用KSC防火墙规则
 * @return 成功返回0,失败返回-1 
 */
int kysec_firewall_enable_rule(char *rule_name);

/*
 * @brief 禁用KSC防火墙规则
 * @param rule_name         防火墙规则名
 * @return 成功返回0,失败返回-1 
 */
int kysec_firewall_disable_rule(char *rule_name);

/*
 * @brief 更新KSC防火墙规则
 * @param rule          防火墙规则结构体指针
 * @return 成功返回0
 *  *      失败返回值：
 *  RULE_EXIST_ERROR  (-13)  规则不存在
 *  NULL_RULE_ERROR  （-12）  规则为空
 *  RULE_NAME_ERROR  （-11）  规则名为空
 *  DIRECTION_ERROR  （-10）  方向错误
 *  NET_MODE_ERROR   （-9）   网络模式错误
 *  ACTION_ERROR     （-8）   操作错误
 *  STATUS_ERROR     （-7）   状态错误
 *  PROTOCOL_ERROR   （-6）   协议错误
 *  LOCAL_IP_ERROR   （-5）   本地IP错误
 *  REMOTE_IP_ERROR  （-4）   远程IP错误
 *  LOCAL_PORT_ERROR （-3）   本地端口错误
 *  REMOTE_PORT_ERROR（-2）   远程端口错误
 *  NORMAL_ERROR     （-1）   规则更新失败
 * @attention 暂定规则名为规则标志符，不可更改
 */
int kysec_firewall_update_rule(KYSEC_FW_RULE *rule);

/*
 * @brief 删除KSC防火墙规则
 * @param rule_name         防火墙规则名
 * @return 成功返回0,失败返回-1 
 */
int kysec_firewall_delete_rule(char *rule_name);

/*
 * @brief 获取KSC防火墙规则
 * @param rule_name KSC防火墙规则名
 * @return 成功返回KYSEC_FW_RULE规则结构体,失败返回NULL
 */
KYSEC_FW_RULE *kysec_firewall_get_rule(char *rule_name);

/*
 * @brief 获取KSC防火墙所有规则
 * @param rules KYSEC_FW_RULE结构体数组指针
 * @return 成功返回获取到的所有规则数目,失败返回-1
 */
int kysec_firewall_get_all_rules(KYSEC_FW_RULE ***rules);

/*
 * @brief 导出KSC防火墙规则
 * @param rules_path  规则导出文件路径
 * @param rules       导出规则名字符串组
 * @param count       导出规则数目
 * @return 成功返回0,失败返回-1 
 */
int kysec_firewall_export_rules(char *rules_path, char *rules[], int count);

/*
 * @brief 导入KSC防火墙规则
 * @param rules_path 导入导出文件路径
 * @return 成功返回导入规则数目,失败返回-1 
 */
int kysec_firewall_import_rules(char *rules_path);

# ifdef  __cplusplus
}
# endif

#endif  /* _KYSEC_FIREWALL_H */
