#ifndef KYSEC_KID_H__
#define KYSEC_KID_H__

#include <stddef.h>
#include <stdint.h>
#include <sys/queue.h>
#include <linux/limits.h>
#include <kysec/db.h>

#ifndef KID_NAME_LEN
#define KID_NAME_LEN 128
#endif

/**
 * @brief kid信息结构
 * 
 */
typedef struct kysec_kid_info_t{
    uint32_t kid;                           //kid值
    char source_name[KID_NAME_LEN];      //源码包名称， 系统资源时为系统等级名称
    char app_name[KID_NAME_LEN];         //应用名称
    uint32_t id_type;                       //id类型
}kysec_kid_info;

/**
 * @brief Construct a new slist head object
 *        kid信息列表
 * 
 */
SLIST_HEAD(kid_info_slist, kid_info_entry);
struct kid_info_entry {
	kysec_kid_info kid;
	SLIST_ENTRY(kid_info_entry) entries;
};
typedef struct kid_info_slist kid_info_list;

/**
 * @brief kid资源结构
 * 
 */
typedef struct kysec_kid_res_t{
    uint32_t kid;                           //kid值
    uint32_t resource_type;                 //资源类型， 0 普通资源， 1 遍历目录， 2 递归遍历目录
    char resource[PATH_MAX];             //资源路径
    char app_name[KID_NAME_LEN];         //应用名称
}kysec_kid_res;

/**
 * @brief Construct a new slist head object
 *        kid资源列表
 * 
 */
SLIST_HEAD(kid_res_slist, kid_res_entry);
struct kid_res_entry {
	kysec_kid_res res;
	SLIST_ENTRY(kid_res_entry) entries;
};
typedef struct kid_res_slist kid_res_list;

/**
 * @brief kid权限控制结构
 * 
 */
typedef struct kysec_kid_priv_t{
    uint32_t sub_kid;            //主体kid
    uint32_t obj_kid;            //客体kid
    uint32_t uid;
    uint32_t privilege;         //权限
    uint32_t type;              //类型， 0 内核策略， 1 核外策略
}kysec_kid_priv;

/**
 * @brief Construct a new slist head object 
 *        kid权限控制列表
 * 
 */
SLIST_HEAD(kid_priv_slist, kid_priv_entry);
struct kid_priv_entry {
	kysec_kid_priv priv;
	SLIST_ENTRY(kid_priv_entry) entries;
};
typedef struct kid_priv_slist kid_priv_list;

/**
 * @brief kid应用联网控制
 * 
 */
typedef struct kysec_kid_netctl_t{
    uint32_t kid;           //kid
    uint32_t priv_type;     //控制类型
    char path[PATH_MAX]; //应用路径
}kysec_kid_netctl;

/**
 * @brief Construct a new slist head object
 *        kid应用联网控制列表
 * 
 */
SLIST_HEAD(kid_netctl_slist, kid_netctl_entry);
struct kid_netctl_entry {
	kysec_kid_netctl netctl;
	SLIST_ENTRY(kid_netctl_entry) entries;
};
typedef struct kid_netctl_slist kid_netctl_list;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 申请一个kid
 * 
 * @param source_name 源码包名称
 * @param app_name 二进制包名称
 * @param section 该字段预留，用于判断生成kid时所属区间
 * @return int < 0 失败
 *             >= 0 kid值
 */
int kysec_kid_apply_for_kid(const char *source_name, const char *app_name, const char *section);

/**
 * @brief 为资源申请一个tmp类kid
 * 
 * @param path 资源路径
 * @return int  < 0 失败
 *             >= 0 kid值
 */
int kysec_kid_apply_for_tmp_kid(const char *path);

/**
 * @brief 添加应用kid信息
 * 
 * @param kid 应用kid基本信息
 * @param file_list 应用文件列表
 * @param size 应用文件列表数量
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_create_kid_data(kysec_kid_info kid, char *file_list[], int size);

/**
 * @brief 提升应用kid信息
 * 
 * @param kid 应用kid基本信息
 * @param file_list 应用文件列表
 * @param size 应用文件列表数量
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_improve_kid_data(uint32_t old_kid, kysec_kid_info kid, char *file_list[], int size);

/**
 * @brief 根据deb包名获取kid
 * 
 * @param app_name 包名
 * @return int < 0 失败
 *             >= 0 kid值
 */
int kysec_kid_get_kid_by_app_name(const char *app_name);

/**
 * @brief 获取应用kid信息
 * 
 * @param kid KID
 * @param info_list 传出，应用kid基本信息列表
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_load_kid_info_list(uint32_t kid, kid_info_list *info_list);

/**
 * @brief 获取应用kid信息
 * 
 * @param kid KID
 * @param info_list 传出，应用kid基本信息列表
 * @param res_list 传出， 应用kid资源列表
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_load_kid_data(uint32_t kid, kid_info_list *info_list, kid_res_list *res_list);

/**
 * @brief 添加kid资源列表
 * 
 * @param kid kid
 * @param type 资源类型  0 普通资源， 1 遍历目录  2 递归遍历目录
 * @param list 待添加资源列表
 * @param size 待添加资源列表长度
 * @return int 0 成功，其他 失败
 */
int kysec_kid_add_kid_res_list(uint32_t kid, int type, char *list[], int size);

/**
 * @brief 删除kid资源列表
 * 
 * @param kid kid
 * @param type 资源类型  0 普通资源， 1 遍历目录  2 递归遍历目录
 * @param list 待添加资源列表
 * @param size 待添加资源列表长度
 * @return int 0 成功，其他 失败
 */
int kysec_kid_remove_kid_res_list(uint32_t kid, int type, char *list[], int size);

/**
 * @brief 获取kid资源列表
 * 
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_res_list(uint32_t kid, kid_res_list *res_list);

/**
 * @brief 获取kid资源列表
 * 
 * @param kid_min KID最小
 * @param kidx_max KID最大
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_res_list_by_kid_range(uint32_t kid_min, uint32_t kid_max, kid_res_list *res_list);

/**
 * @brief 获取kid资源列表
 * 
 * @param kid_min KID最小
 * @param kidx_max KID最大
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_info_list_by_kid_range(uint32_t kid_min, uint32_t kid_max, kid_info_list *info_list);

/**
 * @brief 释放kid应用信息列表内存
 * 
 * @param info_list 待释放kid信息列表
 */
void kysec_kid_free_kid_info_list(kid_info_list *list);

/**
 * @brief 释放kid资源列表内存
 * 
 * @param info_list 待释放kid资源信息列表
 */
void kysec_kid_free_kid_res_list(kid_res_list *list);

/**
 * @brief 创建kid权限管控记录
 * 
 * @param list 权限控制列表
 * @param size 列表数量
 * @return int 0 成功，其他 失败
 */
int kysec_kid_create_kid_privilege(const kysec_kid_priv list[], int size);

/**
 * @brief 更新kid权限管控记录
 * 
 * @param list 权限控制列表
 * @param size 列表数量
 * @return int 0 成功，其他 失败
 */
int kysec_kid_update_kid_privilege(const kysec_kid_priv list[], int size);

/**
 * @brief 删除kid权限管控记录
 * 
 * @param list 权限控制列表
 * @param size 列表数量
 * @return int 0 成功，其他 失败
 */
int kysec_kid_remove_kid_privilege(const kysec_kid_priv list[], int size);

/**
 * @brief 获取kid权限管控记录
 * 
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_privilege(kid_priv_list *list);

/**
 * @brief 根据主体kid获取kid权限管控记录
 * 
 * @param sub_kid 主体kid
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_privilege_by_subject(uint32_t sub_kid, kid_priv_list *list);

/**
 * @brief 根据客体kid获取kid权限管控记录
 * 
 * @param obj_kid 主体kid
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_privilege_by_object(uint32_t obj_kid, kid_priv_list *list);

/**
 * @brief 根据主体kid和客体kid获取kid权限管控记录
 * 
 * @param sub_kid 主体kid
 * @param obj_kid 客体kid
 * @param list 传出权限控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_privilege_by_subject_and_object(uint32_t sub_kid, uint32_t obj_kid, kid_priv_list *list);

/**
 * @brief 同步kid权限策略至内核
 * 
 * @param sync_type 同步类型， 0 添加， 1 删除
 * @param list 待同步数据
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_sync_kid_privilege_to_kernel(int sync_type, kid_priv_list *list);

/**
 * @brief 释放kid列表内存
 * 
 * @param info_list 待释放kid资源信息列表
 */
void kysec_kid_free_kid_priv_list(kid_priv_list *list);

/**
 * @brief 新建kid联网管控记录
 * 
 * @param list 权限控制列表
 * @param size 列表数量
 * @return int 0 成功，其他 失败
 */
int kysec_kid_create_kid_netctl(const kysec_kid_netctl list[], int size);

/**
 * @brief 更新kid联网管控记录
 * 
 * @param list 权限控制列表
 * @param size 列表数量
 * @return int 0 成功，其他 失败
 */
int kysec_kid_update_kid_netctl(const kysec_kid_netctl list[], int size);

/**
 * @brief 删除kid联网管控记录
 * 
 * @param list 权限控制列表
 * @param size 列表数量
 * @return int 0 成功，其他 失败
 */
int kysec_kid_remove_kid_netctl(const kysec_kid_netctl list[], int size);

/**
 * @brief 获取kid联网管控记录
 * 
 * @param list 传出联网控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_netctl(kid_netctl_list *list);

/**
 * @brief 获取指定kid联网管控记录
 * 
 * @param kid kid
 * @param list 传出联网控制列表
 * @return int 0 成功，其他 失败
 */
int kysec_kid_load_kid_netctl_by_kid(uint32_t kid, kid_netctl_list *list);

/**
 * @brief 释放kid列表内存
 * 
 * @param info_list 待释放kid资源信息列表
 */
void kysec_kid_free_kid_netctl_list(kid_netctl_list *list);


#ifdef __cplusplus
}
#endif

#endif

